// Tentakel-VertexShader

$rein float2 reinVpos;   // Reckeck-Ecke xy (0 .. 1)
$rein float4 reinPos[2]; // und TEXCOORD1: Position der Ecken
$rein float4 reinGrafik; // Grafikbereich in Texturpixeln, xy = linksoben, zw = BreiteHöhe
$rein float4 reinFarbe[4]; // bis einschl. TEXCOORD6: Farben der vier Ecken

$raus float4 psPos;
$raus float4 psFarbe;
$raus float2 psTex;
$raus float4 psTexGrenz;
$raus float4 psDxdy; // Tangente für X und Y-Texturrichtung

uniform float2 gQuellGroesse; // Größe der Quelltextur in 1/Pixel
uniform float2 gZielGroesse; // Größe des Zielbereichs, in 1/Pixel, y bereits negiert

$main()
{
  int pzeile = reinVpos.y > 0.5f ? 1 : 0;
  int pspalte = reinVpos.x > 0.5f ? 1 : 0;
  float2 pos = pspalte > 0 ? reinPos[pzeile].zw : reinPos[pzeile].xy;

#if GRAFIK_DIRECTX == 9
  // mit halbem Pixel Offset nach links oben für DX9-Rasterizing Rules
  pos -= float2( 0.5f, 0.5f);  
#endif
  psPos = float4( pos.xy * gZielGroesse.xy * 2.0f + float2( -1.0f, 1.0f), 0.5f, 1.0f);
  
  int findex = int( reinVpos.y * 2.0f + reinVpos.x);
  psFarbe = reinFarbe[findex];

  psTex = (reinGrafik.xy + reinVpos.xy * reinGrafik.zw) * gQuellGroesse.xy;

  // Tangentenvektoren in X und Y berechnen. Daraus leiten wir Rotation und Texturrandweite ab
  float2 tangx[2];
  tangx[0] = reinPos[0].zw - reinPos[0].xy;
  tangx[1] = reinPos[1].zw - reinPos[1].xy;
  float2 tangy[2];
  tangy[0] = reinPos[1].xy - reinPos[0].xy;
  tangy[1] = reinPos[1].zw - reinPos[0].zw;

  // X-Tangente und Y-Tangente einfach mal normiert psschreiben und gucken, wie es aussieht.
  // die Idee mit atan2() und dem interpolierten Winkel war nicht so gut, weil die Interpolation über +-PI springt
  psDxdy.xy = normalize( tangx[pzeile]);
  psDxdy.zw = normalize( tangy[pspalte]);

  // Größe des Tex-Randbereiches ermitteln. Standard ist ein halber Pixel, bei verkleinerter Darstellung 
  // und entsprechender MipMap-Nutzung muss der Grenzbereich entsprechend größer werden
  float grx = min( length( tangx[0]), length( tangx[1]));
  float gry = min( length( tangy[0]), length( tangy[1]));

  float randGrenze = min( grx / reinGrafik.z, gry / reinGrafik.w);
  randGrenze = 0.5f / clamp( randGrenze, 1.0f/16.0f, 1.0f);
  psTexGrenz = (reinGrafik.xyxy + float4( 0, 0, 1, 1) * reinGrafik.zwzw + float4( 1, 1, -1, -1) * randGrenze) * gQuellGroesse.xyxy;
}
